import * as fs from 'fs/promises';
import { EnumSingleGenerator } from './enum-generator.js';
import { ERROR_CODES, INDE_TS_TYPES_MAP, CustomError } from './various.js';
export class ClassGenerator {
    /**
     * Logica del nome classe: se nel nome fornito son presenti maiuscole isolate, si ipotizza camelcase, e quindi tutto resta invariato, tranne la prima lettera che viene messa in maiuscolo
     * Se non sono presenti maiuscole isolate, il nome viene messo tutto in minuscolo, tranne la prima lettera, che resta in maiuscolo
     * PIPPO => Pippo
     * pippo => Pippo
     * PIPPOPLUTO => Pippopluto
     * pippopluto => Pippopluto
     * pippoPluto => PippoPluto
     *
     */
    get className() {
        if (!this.params.normalizeClassNames) {
            return this.suppliedName;
        }
        else {
            let isCamelCase = false;
            const strLength = this.suppliedName.length;
            let finalString = this.suppliedName;
            for (let i = 1; i < strLength - 1; i++) {
                if (this.isUppercase(this.suppliedName[i]) && !this.isUppercase(this.suppliedName[i - 1]) && !this.isUppercase(this.suppliedName[i + 1])) {
                    //abbiamo una maiuscola isolata, 
                    isCamelCase = true;
                }
            }
            if (!isCamelCase) {
                //non ci sono maiuscole da salvare, metto tutto in minuscolo 
                finalString = finalString.charAt(0).toUpperCase() + finalString.substring(1).toLowerCase();
            }
            else {
                finalString = finalString.charAt(0).toUpperCase() + finalString.substring(1);
            }
            return finalString;
        }
    }
    setKeys(keys) {
        if (!!keys && keys.length > 0) {
            this.arPrimaryKeys = keys;
        }
    }
    /**
     * Logica per il nome file: il nome viene sempre messo in minuscolo, ma se viene rilevato camelcase, le parole vengono spezzate con dei trattini
     * viene aggiunto .model.ts alla fine
     * PIPPO => pippo.model.ts
     * pippo => pippo.model.ts
     * PIPPOPLUTO => pippopluto.model.ts
     * pippopluto => pippopluto.model.ts
     * pippoPluto => pippo-pluto.model.ts
     */
    get fileName() {
        if (!this.params.normalizeClassFilesNames) {
            return `${this.suppliedName}.ts`;
        }
        else {
            let isCamelCase = false;
            const strLength = this.suppliedName.length;
            let finalString = this.suppliedName;
            let j = 1;
            for (let i = j; i < strLength - 1; i++) {
                if (this.isUppercase(this.suppliedName[i]) && !this.isUppercase(this.suppliedName[i - 1]) && !this.isUppercase(this.suppliedName[i + 1])) {
                    //abbiamo una maiuscola isolata, 
                    isCamelCase = true;
                    finalString = `${finalString.substring(0, j)}-${finalString.substring(j)}`;
                    j++;
                }
                j++;
            }
            return `${finalString.toLowerCase()}.model.ts`;
        }
    }
    isUppercase(letter) {
        return (letter === letter.toUpperCase());
    }
    getInitValueForType(type) {
        let returnValue = '';
        switch (type) {
            case "Date":
                // returnValue = " = new Date()";
                returnValue = " = new Date(0)";
                break;
            case "string":
                returnValue = ` = ''`;
                break;
            default:
                returnValue = '';
        }
        return returnValue;
    }
    constructor(name, config) {
        //proprietà della classe
        this.arProperties = [];
        //nome fornito
        this.suppliedName = '';
        //array di importazioni da aggiungere
        this.arImports = [];
        this.arPrimaryKeys = [];
        this.suppliedName = name;
        this.params = config;
        ;
    }
    addPrimaryKey(name, type, required = false, privateField = 'public') {
        this.arPrimaryKeys.push(name);
        this.addProperty(name, type, required, privateField, true);
        return this;
    }
    addProperty(name, type, required = false, accessibility = 'public', isPrimary = false) {
        if (!!name && name.length > 0 && !!type && type.length > 0 && required !== null && required !== undefined) {
            this.arProperties.push({
                name,
                type,
                required,
                accessibility,
                isPrimary
            });
        }
        else {
            console.warn(`Error, property ${name} ignored, not all parameters supplied`);
        }
        return this;
    }
    /**
    * Aggiunge un elemento all'elenco di importazioni (se l'elemento è già stato aggiunto, viene ignorato)
    * @param tokenName
    * @param path
    */
    addImport(tokenName, path) {
        let added = false;
        if (!!tokenName && tokenName.length > 0 && !!path && path.length > 0) {
            this.arImports.forEach(el => {
                if (el.sourcePathName == path) {
                    // abbiamo già un import con lo stesso path, ma dobbiamo controllare che lo stesso token non sia già importato
                    if (!el.tokens.includes(tokenName)) {
                        el.tokens.push(tokenName);
                        added = true;
                    }
                    else {
                        //token già presente, non faccio niente
                        added = true;
                    }
                }
            });
            if (!added) {
                this.arImports.push({ sourcePathName: path, tokens: [tokenName] });
            }
        }
    }
    /**
     * Restituisce il contenuto del file finito sotto forma di stringa
     * @returns
     */
    getFileContentString() {
        const start = `export class ${this.className} ${this.getInheritance()}{\n`;
        const end = `}`;
        const properties = this.getPropertiesString();
        const imports = this.getImportsString();
        const constructor = this.getConstructor();
        const initMethod = this.getInitMethod();
        const initAllMethod = this.getInitAllMethod();
        const factoryMethod = this.getFactoryMethod();
        const classNameMethod = this.getClassNameMethod();
        const remoteNameMethod = this.getRemoteNameMehtod();
        const accessorMethods = this.getAccessorMethods();
        const keyDescriptorMethod = this.getKeyDescriptorMethod();
        return imports +
            start +
            properties +
            constructor +
            initMethod +
            initAllMethod +
            keyDescriptorMethod +
            factoryMethod +
            classNameMethod +
            remoteNameMethod +
            accessorMethods +
            end;
    }
    //
    getInheritance() {
        let extendPart = '';
        let implementsPart = '';
        if (!!this.params.extendClass && this.params.extendClass?.name?.length > 0 && this.params.extendClass?.importPath?.length > 0) {
            extendPart = `extends ${this.params.extendClass.name} `;
            this.addImport(this.params.extendClass.name, this.params.extendClass.importPath);
        }
        if (!!this.params.extendInterfaces && this.params.extendInterfaces.length > 0) {
            implementsPart = this.params.extendInterfaces
                .map(el => el.name)
                .reduce((acc, el, idx) => {
                if (el?.length > 0) {
                    this.addImport(el, this.params.extendInterfaces[idx].importPath);
                    return `${acc}, ${el}`;
                }
                else {
                    return '';
                }
            });
            implementsPart = `implements ${implementsPart}`;
        }
        return extendPart + implementsPart;
    }
    getPropertiesString() {
        const startRegion = this.params.regionAnnotations ? '\t//#region PROPERTIES\n' : '';
        const endRegion = this.params.regionAnnotations ? '\t//#endregion\n' : '';
        let properties = '';
        if (!!this.arProperties && this.arProperties.length > 0) {
            properties = this.arProperties
                .map((el) => {
                return `\t${el.accessibility} ${el.name}${el.required ? '' : '?'}: ${el.type}${(this.params.initProperties && this.params.initPropertiesMode == 'normal') ? this.getInitValueForType(el.type) : ''};\n`;
            })
                .reduce((el, acc) => acc + el);
            properties = startRegion + properties + endRegion + '\n' + '\n';
        }
        return properties;
    }
    getImportsString() {
        return this.arImports.length > 0 ? this.arImports
            .map(el => {
            return `import { ${el.tokens.map((token, idx) => `${token}, `).reduce((stringToken, acc) => (stringToken + acc)).slice(0, -2)} } from '${el.sourcePathName}'\n`;
        })
            .reduce((el, acc) => el + acc)
            .concat('\n\n') : '';
    }
    getConstructor() {
        let finalString = '';
        if (this.params.initProperties && this.params.initPropertiesMode == 'constructor') {
            const start = `\tconstructor() {\n`;
            const end = `\t}`;
            const properties = this.arProperties.length > 0 ? this.arProperties
                .map(elProp => `\t\tthis.${elProp.name}${this.getInitValueForType(elProp.type)};\n`)
                .reduce((el, acc) => acc + el) : '';
            finalString = start + properties + end + '\n' + '\n';
        }
        return finalString;
    }
    getInitMethod() {
        let finalString = '';
        if (this.params.initProperties && this.params.initPropertiesMode == 'initMethod') {
            const start = `\tinit(): this {\n`;
            const end = `\t\treturn this\n\t}`;
            const properties = this.arProperties.length > 0 ? this.arProperties
                .map(elProp => `\t\tthis.${elProp.name}${this.getInitValueForType(elProp.type)};\n`)
                .reduce((el, acc) => acc + el) : '';
            finalString = start + properties + end + "\n" + "\n";
        }
        return finalString;
    }
    getKeyDescriptorMethod() {
        let finalString = '';
        if (this.params.getKeyDescriptorMethod) {
            const start = `\tgetPrimaryKeys(): string[]{\n`;
            const end = '\t}';
            const body = `\t\treturn [${this.arPrimaryKeys.length > 0 ? this.arPrimaryKeys.map(el => `"${el}"`).reduce((acc, el) => `${acc},${el}`) : ''}]\n`;
            finalString = start + body + end + '\n' + '\n';
        }
        return finalString;
    }
    getInitAllMethod() {
        let finalString = '';
        if (this.params.getInitAllMethods) {
            const start = `\tinitAll(): this {\n`;
            const end = `\t\treturn this\n\t}`;
            const properties = this.arProperties.length > 0 ? this.arProperties
                .map(elProp => `\t\t//@ts-ignore\n\t\tthis.${elProp.name} = undefined;\n`)
                .reduce((el, acc) => acc + el) : '';
            finalString = start + properties + end + "\n" + "\n";
        }
        return finalString;
    }
    getAccessorMethods() {
        const startRegion = this.params.regionAnnotations ? '\t//#region ACCESSORS\n' : '';
        const endRegion = this.params.regionAnnotations ? '\t//#endregion \n' : '';
        let final = '';
        if (this.params.propertiesGetMethods || this.params.propertiesSetMethods) {
            final = this.arProperties.map((elProp, idx) => {
                const finalPropName = elProp.name.substring(1, elProp.name.length);
                const getMethod = this.params.propertiesGetMethods ? `\tpublic get ${finalPropName}(){return this.${elProp.name}}\n` : ``;
                const setMethod = this.params.propertiesSetMethods ? `\tpublic set ${finalPropName}(value){this.${elProp.name} = value}\n` : ``;
                return getMethod + setMethod + ((idx < this.arProperties.length - 1) ? '\n' : '');
            })
                .reduce((el, acc) => el + acc);
        }
        return startRegion + final + endRegion + '\n';
    }
    getFactoryMethod() {
        if (this.params.getFactoryMethod) {
            return `\tgetFactory(): typeof ${this.className}{
        return ${this.className}
    }\n\n`;
        }
        else {
            return '';
        }
    }
    getClassNameMethod() {
        if (this.params.getFactoryMethod) {
            return `\tgetClassName(): string{
        return '${this.className}'
    }\n\n`;
        }
        else {
            return '';
        }
    }
    getRemoteNameMehtod() {
        if (this.params.getFactoryMethod) {
            return `\tgetRemoteName(): string{
        return '${this.suppliedName}'
    }\n\n`;
        }
        else {
            return '';
        }
    }
    /**
     * Salva la classe nel percorso specificato
     * @param path
     * @returns
     */
    saveOnFileSystem(path) {
        const fileContent = this.getFileContentString();
        const finalPath = `${path}${path.endsWith('/') ? '' : '/'}${this.fileName}`;
        return fs.writeFile(finalPath, fileContent)
            .catch(err => Promise.reject(new CustomError(err, ERROR_CODES.ERR_SAVING_MODEL_FILE)));
    }
    /**
    * converte il tipo proveniente da inde in tipo typescript (se il tipo non è standard, si suppone sia un enum)
    * @param indeType tipo proveniente da inde
    * @returns
    */
    static convertToTsType(indeType) {
        let finalType = null;
        let arTsTypes = Object.keys(INDE_TS_TYPES_MAP);
        let isEnum = false;
        arTsTypes.forEach((key, idx, other) => {
            if (INDE_TS_TYPES_MAP[key].includes(indeType))
                finalType = key;
        });
        if (finalType == null) {
            // si suppone che il tipo sia un enum, tolgo i punti e prendo solo l'ultima parte
            finalType = EnumSingleGenerator.normalizeEnumName(indeType);
            isEnum = true;
        }
        return { finalType, isEnum };
    }
}
//# sourceMappingURL=class-generator.js.map